from pyspark.sql import DataFrame

# from pyspark.sql.column import Column
from pyspark.sql.functions import col, lit, struct, when
from pyspark.sql.types import (
    StringType,
    BinaryType,
    TimestampType,
    DoubleType,
    LongType,
)

import trend_etl.schema.fields as fields
from trend_etl.functions.index import (
    jobid_2_timestamp,
    parse_subreddits,
    get_collected_day,
    is_scientific,
)


# arguments:
#   archive_df: the dataframe from the archives
#   provenance: short description of the origin (kafka, s3 archive, etc.)
def from_archive_get_bronze_subreddits_df(archive_df: DataFrame) -> DataFrame:
    return archive_df.select(
        col(fields.archive_value_field)
        .cast(BinaryType())
        .alias(fields.bronze_value_field),
        lit("archive/reddit/subreddits").alias(fields.bronze_provenance_field),
        jobid_2_timestamp(fields.archive_job_id_field).alias(
            fields.bronze_timestamp_field
        ),
    ).withColumn(
        fields.collected_day_field,
        get_collected_day(col(fields.bronze_timestamp_field)),
    )


#
# arguments:
#   input_df: dataframe received from reading a kafka topic
#   target_topic: source topic from kafka
# return: a dataframe in the schema expected for bronze tables (for all reddit data kinds)
def from_kafka_get_bronze_subreddits_df(
    input_df: DataFrame, target_topic: str
) -> DataFrame:
    return input_df.select(
        col(fields.kafka_value_field).alias(fields.bronze_value_field),
        lit(f"kafka/{target_topic}").alias(fields.bronze_provenance_field),
        col(fields.kafka_timestamp_field).alias(fields.bronze_timestamp_field),
        get_collected_day(col(fields.kafka_timestamp_field)).alias(
            fields.collected_day_field
        ),
    )


# arguments:
#   streaming_bronze: streaming dataframe from bronze table
#   subreddit_idents: groups uuid idents table
# return: dataframe in the schema of facebook_groups
def from_bronze_get_subreddits_df(
    streaming_bronze: DataFrame,
    subreddit_idents: DataFrame,
) -> DataFrame:
    parsed_df = streaming_bronze.select(
        parse_subreddits(col(fields.bronze_value_field).cast(StringType())).alias(
            "value"
        ),
        col(fields.bronze_timestamp_field),
        col(fields.collected_day_field),
    )
    # NOTE: This filter is necessary because the bronze table
    #       for subreddits got kind=Listing documents appended.
    #       For more info see the below bug report:
    #       https://trend-community.monday.com/boards/3371582960/pulses/7142820924
    filtered = parsed_df.where(col("value.kind") == lit("t5"))
    flattened_df = filtered.select(
        col("value.data.*"),
        col(fields.bronze_timestamp_field),
        col(fields.collected_day_field),
    )

    # NOTE: left outer join (below) will cause any nulls to be
    #       detected as a downstream error.

    # join with subreddit_idents
    subreddit_idents_df = (
        flattened_df.join(
            subreddit_idents.withColumnRenamed(fields.idents_source_id_field, "name"),
            "name",
            "left_outer",
        )
        .withColumn(fields.source_keys_field, struct(col("name").alias("name")))
        .drop("name", "id")
    )

    subreddits_df = subreddit_idents_df.withColumn(
        fields.reddit_submissions_created_at,
        when(
            is_scientific(col("created_utc")),
            col("created_utc").cast(DoubleType()).cast(LongType()),
        )
        .otherwise(col("created_utc").cast(LongType()))
        .cast(TimestampType()),
    ).drop("created", "created_utc")

    return subreddits_df
